
#version 330 core

struct Material 
{
    sampler2D texture_diffuse;
	sampler2D texture_diffuse1;
    sampler2D texture_specular;
	sampler2D texture_specular1;
	vec3 diffuseColor;
	vec3 specularColor;
    float shininess;
}; 

struct DirectionalLight 
{
    vec3 direction;
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
	vec3 position;
};

in VS_OUT 
{
    vec3 FragmentPosition;
    vec3 Normal;
    vec2 TexCoords;
    vec4 FragmentPositionInLightSpace;
} fragShaderIn;

out vec4 glFragColor;

uniform vec3 cameraPosition;
uniform DirectionalLight directionalLight;
uniform Material material;
uniform bool hasTexture;
uniform sampler2D shadowMap;

float shadowCalculation(vec4 fragPosLightSpace)
{
    // perform perspective divide
    vec3 projCoords = fragPosLightSpace.xyz / fragPosLightSpace.w;
    // transform to [0,1] range
    projCoords = projCoords * 0.5 + 0.5;
    // get closest depth value from light's perspective (using [0,1] range fragPosLight as coords)
    float closestDepth = texture(shadowMap, projCoords.xy).r; 
    // get depth of current fragment from light's perspective
    float currentDepth = projCoords.z;
    // calculate bias (based on depth map resolution and slope)
    vec3 normal = normalize(fragShaderIn.Normal);
    vec3 lightDirection = normalize(directionalLight.position - fragShaderIn.FragmentPosition);
    float bias = max(0.05f * (1.0f - dot(normal, lightDirection)), 0.005f);
    // check whether current fragment position is in shadow
    // float shadow = currentDepth - bias > closestDepth  ? 1.0 : 0.0;
    
	// PCF
    float shadow = 0.0f;
    vec2 texelSize = 1.0f / textureSize(shadowMap, 0);
    for(int x = -1; x <= 1; ++x)
    {
        for(int y = -1; y <= 1; ++y)
        {
            float pcfDepth = texture(shadowMap, projCoords.xy + vec2(x, y) * texelSize).r; 
            shadow += currentDepth - bias > pcfDepth  ? 1.0f : 0.0f;        
        }    
    }
    shadow /= 9.0;
    
    // keep the shadow at 0.0 when outside the far_plane region of the light's frustum.
    if(projCoords.z > 1.0f)
        shadow = 0.0f;
        
    return shadow;
}

void main()
{    
    // calculate normals and view direction
    vec3 normal = normalize(fragShaderIn.Normal);
    vec3 viewDirection = normalize(cameraPosition - fragShaderIn.FragmentPosition);
    
	// calculate directional light now
	vec3 lightDirection = normalize(-directionalLight.direction);

    // diffuse part
    float diffuseFactor = max(dot(normal, lightDirection), 0.0f);

    // specular part
    vec3 reflectDirection = reflect(-lightDirection, normal);
    float specularFactor = pow(max(dot(viewDirection, reflectDirection), 0.0f), material.shininess);

    // put it together, include texture information
	vec3 ambientColor;
	vec3 diffuseColor;
	vec3 specularColor;
	vec3 color;

	if(hasTexture)
	{
		color = texture(material.texture_diffuse1, fragShaderIn.TexCoords).rgb;
		ambientColor = directionalLight.ambient   * color;
		//ambientColor = directionalLight.ambient;
		diffuseColor = directionalLight.diffuse   * diffuseFactor  * color;
		specularColor = directionalLight.specular * specularFactor * vec3( texture( material.texture_specular1, fragShaderIn.TexCoords ) );
	}
	else
	{
		// material has no texture. use material's colors instead
		//ambientColor = vec3(0.0f);
		color = material.diffuseColor;
		ambientColor = directionalLight.ambient   * color;
		diffuseColor = directionalLight.diffuse   * diffuseFactor  * material.diffuseColor;
		specularColor = directionalLight.specular * specularFactor * material.specularColor;
	}

	float shadow = shadowCalculation(fragShaderIn.FragmentPositionInLightSpace); 
    vec3 resultColor = ambientColor + ( (1.0 - shadow) * (diffuseColor + specularColor));
	//vec3 resultColor = (ambientColor + (1.0 - shadow) * (diffuseColor + specularColor) ) * color;

    glFragColor = vec4(resultColor, 1.0f);
}

